
package masterthesis;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Scanner;

/**5
 * 
 *
 * @author Amanda
 */

/* The medical compound splitter splits compound medical words. Use with main 'MasterThesis'. 
Splitted words are marked with a | in the result file, but the class preprocess can be used to replace them.
file_decom includes the word list that the program uses to split compound words.
file_input is the input file
file_name1 is the output file
The medical word list includes 9358 words and were retrieved from www.medicinskordbok.se and from Linköping University (Semantic Mining Multilingual Medical Dictionary)
*/
public class MedDeCompounder {
    BufferedReader br;
    Scanner sc;
    PrintWriter pr;
    ArrayList<ArrayList<String>> inputList;
    ArrayList<String>medList;
    
    File file_decom, file_input, file_name1;
    
    public MedDeCompounder(){
        medList = new ArrayList<> ();
        inputList = new ArrayList<> ();
        
        //Files
        file_decom = new File("medicalWords.txt"); // List of medical words
        file_input = new File ("icd_preprocessed.txt"); // Input file
        file_name1= new File ("icd_deCompounded.txt");   // Output file
    }
    /*
    This function just calls the other functions. 
    */
    public void connect() throws Exception{
            readFileMedList(medList);
            readTheInputFile();
            printToFile(); 
    }
    /*
    Read the medical wordlist and put it into an ArrayList. 
    */
    public void readFileMedList(ArrayList<String> medList) throws Exception{
        br = new BufferedReader(new FileReader(file_decom));
        String line = "";
        while((line = br.readLine())!= null){
            medList.add(line.trim());
        }
    }
    
    /*
    Reads the input-file, like icd or sct files. 
    */
    public void readTheInputFile() throws Exception{
        br = new BufferedReader(new FileReader(file_input));
        String line = "";
        while((line = br.readLine())!= null){
            /*
            The code over reads the lines from the input-files, and the code
            under splits the lines into words. The function "matchWords" compare
            the word from the inputfile with the words from the medical list. If
            the input-word contains the medical word, the function returns a new 
            splitted word and puts it into an ArrayList called inputList.
            */
            inputList.add(new ArrayList<>());
            sc = new Scanner(line);
            while(sc.hasNext()){
                String word = matchWords(sc.next());
                inputList.get(inputList.size()-1).add(word);
            }
        }
    }

    /*
    Check if the input-word contains some medical words. If so, it puts the 
    contained medical words into an ArrayList. Some input-words can contain more 
    than one word with different length, like "Mag-tarmkanalen", which contains 
    the medical words "anal" and "kanal".
    */    
    public String matchWords(String icdword){ 
        ArrayList<String> tempList = new ArrayList<>();
        for(int j = 0; j < medList.size();j++){
            if(containing(icdword, medList.get(j))){
                tempList.add(medList.get(j));
            }
        }
        if(!tempList.isEmpty()){
            /*
            This is an inbuilt function (code below) which sorts the temporary 
            list with the containing words. It sorts the list after the length 
            of the words. The words with most characters will be in the top of 
            the list.
            */
        Comparator<String> x = (String o1, String o2) -> {
            if(o1.length() > o2.length())
                return -1;
            if(o2.length() > o1.length())
                return 1;
            return 0;
        };
        
        Collections.sort(tempList,  x);
        String word = icdword.replace(tempList.get(0),"|"+tempList.get(0)+"|");
        return word;
        }else{
            return icdword;
        }
        
    }
    /*
    A boolean function that returns true or false depending on if the input-word
    contains the medical word. 
    */
    public boolean containing (String icdWord, String medWord){
        return icdWord.contains(medWord);
    }
    
    public void printToFile() throws FileNotFoundException{
        pr = new PrintWriter(file_name1);
        for(int i = 0;i<inputList.size();i++){
            for(int j = 0;j<inputList.get(i).size();j++){
                pr.print(inputList.get(i).get(j)+" ");
            }
            pr.println();
        }
        pr.close();
    }    
}
